/*
 * Author: Jon Trulson <jtrulson@ics.com>
 * Copyright (c) 2015 Intel Corporation.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include <iostream>
#include <stdexcept>
#include <string>

#include "maxsonarez.hpp"

using namespace std;
using namespace upm;

MAXSONAREZ::MAXSONAREZ(int pin, float aref)
{
    if (!(m_aio = mraa_aio_init(pin))) {
        throw std::invalid_argument(std::string(__FUNCTION__) +
                                    ": mraa_aio_init() failed, invalid pin?");
        return;
    }

    m_aRes = (1 << mraa_aio_get_bit(m_aio));
    m_aref = aref;

    // volt's per inch of this sensor
    m_vI = (m_aref / MAXSONAREZ_RES);
}

MAXSONAREZ::~MAXSONAREZ()
{
    mraa_aio_close(m_aio);
}

int
MAXSONAREZ::inches()
{
    int val = mraa_aio_read(m_aio);
    if (val == -1) {
        return -1;
    }
    float volts = float(val) * (m_aref / m_aRes);

    return int(volts / m_vI);
}

float
MAXSONAREZ::getDistance()
{
    return inches() * 2.54;
}
